#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import os, string, sys
import decimal
import gettext
import unicodedata
gettext.install("gimp20", gimp.locale_directory, unicode=True)
import math, random, os, sys

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   The GPL v3 licence is available at: https://www.gnu.org/licenses/gpl-3.0.en.html


# Ver.1 init relase by Brian Andersen https://gimplearn.net/viewtopic.php?f=3&t=744&hilit=Gold+Coin+Tutorial
# Ver.2 MareroQ: add text-along-path-04 by Ofnuts
# Ver.3 MareroQ: add: national characters support (utf-8), creating paths by plugin (no need to import)
# Ver.4 MareroQ: add options: "Make stars?", unnecessary brushes have been removed, cosmetic with sliders and font name

#Start text-along-path-04 by Ofnuts ############################################################################
v,r,l=gimp.version
use28=(v==2) and (r>6)

debug=False
def trace(s):
	if debug:
		print s

BASELINE=0 	# The baseline
TOP=1		# The top of the character box	
BOTTOM=2	# The bottom of the character box
MIDDLEBOX=3	# The middle of the character box
TOPCAPS=4	# The top of uppercase characters
MIDDLECAPS=5	# The middle of uppercase characters

# Characters that won't produce a path (space, etc...)
blankCharacters=' '

TEXT_JUSTIFY_LEFT=0
TEXT_JUSTIFY_RIGHT=1
TEXT_JUSTIFY_CENTER=2
TEXT_JUSTIFY_FILL=3

#----------------------------------------------------
# Enhanced path Stroke
#----------------------------------------------------
class DirectionStroke:
			
	def __init__(self,stroke):
		self.stroke=stroke
		points,closed=self.stroke.points
		bezierCurves=(len(points)/3)-1

		self.precision=.05/bezierCurves

		self.length=stroke.get_length(self.precision)

	def computeThetaWithSlope(self,fromPoint,toPoint,slope):
		dX=toPoint[0]-fromPoint[0]
		dY=toPoint[1]-fromPoint[1]
		
		if math.fabs(dX) < .001:
			theta=math.copysign(math.pi/2,dY)
		else:
			theta=math.atan(slope)
			if dX<0:
				theta=theta+math.pi
		return theta % (math.pi*2)
		
	
	# Facade method, returns only if point is OK
	def getRawPointAtDist(self,dist):
		x,y,slope,valid=self.stroke.get_point_at_dist(dist,self.precision)
		if not valid: # Normally quite unlikely, since we avoid the cases where this could happen
			raise Exception('No point found at %6.4f in stroke %s with length %6.4f' % (dist,self.stroke.ID,self.length))
		return x,y,slope

	def getPointAtDist(self,dist):
		# Now process regular case
		x,y,slope=self.getRawPointAtDist(dist)

		delta=.5
		toX,toY,s=self.getRawPointAtDist(dist+delta) 
		theta=self.computeThetaWithSlope([x,y],[toX,toY],slope)

		return (x,y,theta)

#----------------------------------------------------
# Character in text
#----------------------------------------------------
class Character(object):
	def __init__(self,character,width,height):
		self.character=character
		self.width=width
		self.height=height
		self.path=None
		self.kerning=0
		self.position=0
		self.first=False
		self.last=False
		
	def __str__(self):
		if self.path:
			return "<'%s' (%d,%d) @%3.2f, %d stroke(s)>" % (self.character,self.width,self.height,self.position,len(self.path.strokes))
		else:
			return "<'%s' (%d,%d) @%3.2f, (no path)>" % (self.character,self.width,self.height,self.position)
			
#----------------------------------------------------
# Text to work on
#----------------------------------------------------
class Text(object):
	
	def __init__(self,text,fontName,fontSize,
			layout=TEXT_JUSTIFY_CENTER, pivotYChoice=BASELINE, 
			useKerning=True, extraSpacing=0, verticalAdjust=0,
			keepPaths=False, keepUpright=False,
			wiggleXPercent=0, wiggleYPercent=0, wiggleTheta=0):
		self.text=unicode(text,'utf-8','strict')
		self.fontName=fontName
		self.fontSize=fontSize
		self.pivotYChoice=pivotYChoice
		self.useKerning=useKerning
		self.extraSpacing=extraSpacing
		self.verticalAdjust=verticalAdjust
		self.keepUpright=keepUpright
		self.layout=layout
		self.keepPaths=keepPaths
		self.wiggleXPercent=wiggleXPercent
		self.wiggleYPercent=wiggleYPercent
		self.wiggleTheta=wiggleTheta
		
		self.pivotY=None
		self.characters=[]
		self.workImage=gimp.Image(int(fontSize*4),int(fontSize*4), RGB)
		self.workImage.disable_undo()
		
	def __del__(self):
		gimp.delete(self.workImage)

	def extents(self,text):
		return pdb.gimp_text_get_extents_fontname(text, self.fontSize, PIXELS, self.fontName)
		
	def textPath(self,text):
		if len(text) == 1 and text in blankCharacters:
			path=None
		else:
			l=pdb.gimp_text_fontname(self.workImage, None, 0, 0, text, 0, True, self.fontSize, PIXELS, self.fontName)
			path=pdb.gimp_vectors_new_from_text_layer(self.workImage,l)
			self.workImage.remove_layer(l)
		return path
		
	def initializeCharacter(self,c):
		width,height,ascent,descent=self.extents(c)
		addedChar=Character(c,width,height)
		addedChar.path=self.textPath(c)
		# compute kerning unless first character
		if self.useKerning and len(self.characters) > 0:
			rightChar=self.characters[-1]
			kw,kh,ka,kd=self.extents(rightChar.character+addedChar.character)
			addedChar.kerning=kw-(addedChar.width+rightChar.width)
			if addedChar.kerning !=0:
				trace('Kerning %c -> %c: %3.2f' % (rightChar.character,addedChar.character,addedChar.kerning))

		self.characters.append(addedChar)
	
	def initializeCharacters(self):
		trace('Text: %s' % self.text)
		for c in self.text:
			self.initializeCharacter(c)
			
		self.characters[0].first=True
		self.characters[-1].last=True
		lastChar=self.characters[-1]
			
	def computePivotY(self):
		sampleText='X'
		# obtain path for sample text
		path=self.textPath(sampleText)
		
		width,height,ascent,descent=self.extents(sampleText)
		
		# Gather all Y values (anchors and handles) in all strokes for the bounding box 
		allY=[stroke.points[0][1::2] for stroke in path.strokes] # List of lists
		allY=[y for stroke in allY for y in stroke] # flattened
		minY=min(allY)
		maxY=max(allY)
		
		self.wiggleYMax=height
		topCaps=minY+(maxY-ascent)
		# compute all possible pivotY and keep the good one (easier to debug)
		pivot=[0,0,0,0,0,0]
		pivot[BASELINE]=ascent
		pivot[TOP]=0
		pivot[BOTTOM]=height
		pivot[MIDDLEBOX]=height/2.
		pivot[MIDDLECAPS]=(topCaps+ascent)/2.
		pivot[TOPCAPS]=topCaps
		#trace(pivot)
		self.pivotY=pivot[self.pivotYChoice]+self.verticalAdjust
		
	# Lay out the characters on the stroke, by computing their.position coordinate
	def layoutOnStroke(self,stroke):
		# Plain text width
		plainTextWidth=0
		for c in self.characters:
			plainTextWidth=plainTextWidth+c.width+c.kerning
		
		# Check fit on stroke
		if self.layout != TEXT_JUSTIFY_FILL:
			# Check with requested extra spacing if not justified
			textWidth=plainTextWidth+self.extraSpacing*(len(self.characters)-1)
			if textWidth > stroke.length:
				raise Exception('Text width (%3.2f) larger than path stroke length (%3.2f)' % (textWidth,stroke.length))
			extraSpacing=self.extraSpacing
		else: # TEXT_JUSTIFY_FILL
			extraSpacing=(stroke.length-plainTextWidth)/(len(self.characters)-1)
			textWidth=stroke.length
		self.wiggleXMax=textWidth/len(self.characters)
		# Compute offset of left character
		if self.layout==TEXT_JUSTIFY_LEFT:
			offset=0
		elif self.layout==TEXT_JUSTIFY_RIGHT:
			offset=stroke.length-textWidth
		elif self.layout==TEXT_JUSTIFY_CENTER:
			offset=(stroke.length-textWidth)/2.
		else: #TEXT_JUSTIFY_FILL, normally
			offset=0
		
		trace('Plain text width: %3.2f, stroke length: %3.2f, start offset: %3.2f, extraSpacing: %3.2f, actual text width: %3.2f ' % (plainTextWidth,stroke.length,offset, extraSpacing, textWidth))
		
		# set.position for each character
		for c in self.characters:
			offset=offset+c.width/2.+c.kerning
			c.position=offset
			offset=offset+c.width/2.+extraSpacing
			
	# compute final pos for character
	def computeFinalPos(self,c,stroke):
		x,y,slope=stroke.getPointAtDist(c.position)
		if self.keepUpright:
			tilt=0
		else:
			tilt=slope*180/math.pi
		
		wiggleXRange=self.wiggleXMax*self.wiggleXPercent/100.
		wiggleYRange=self.wiggleYMax*self.wiggleYPercent/100.

		wx=random.uniform(-wiggleXRange,wiggleXRange)
		wy=random.uniform(-wiggleYRange,wiggleYRange)
		wtilt=random.uniform(-self.wiggleTheta,self.wiggleTheta)

		return x+wx*math.cos(slope)-wy*math.sin(slope),y+wy*math.cos(slope)+wx*math.sin(slope),tilt+wtilt
		
	# copy path (possibly to different image)
	def copyPath(self,sourcePath,name,img):
		newPath=pdb.gimp_vectors_new(img,name)
		pdb.gimp_image_add_vectors(img,newPath,0)
		self.copyStrokes(sourcePath,newPath)
		return newPath
		
	def copyStrokes(self,sourcePath,targetPath):
		for s in sourcePath.strokes:
			points,closed=s.points
			sid = pdb.gimp_vectors_stroke_new_from_points(targetPath,0, len(points),points, closed)
		
		
	def moveCharacterToStroke(self,c,stroke,img):
		if not c.path:
			return # nothing to do on blank characters
		# copy character path (defined on work image) to final image
		path=self.copyPath(c.path,"'"+c.character+"'",img)
		path.visible=False;
		x,y,tilt=self.computeFinalPos(c,stroke)
		trace('"%s": actual: %3.2f' % (c,x))
		for s in path.strokes:
			 s.rotate(c.width/2.,self.pivotY,tilt)
			 s.translate(x-c.width/2.,y-self.pivotY)
		self.copyStrokes(path,self.finalPath)
		if not self.keepPaths:
			pdb.gimp_image_remove_vectors(img,path)
			 
	def moveCharactersToStroke(self,stroke,img,pathName):
		self.finalPath=pdb.gimp_vectors_new(img,"'%s' over <%s>" % (self.text,pathName))
		self.finalPath.visible=True
		pdb.gimp_image_add_vectors(img,self.finalPath,0)

		trace("Max wiggle X,Y: %3.2f,%3.2f" % (self.wiggleXMax,self.wiggleYMax))
		for c in self.characters:
			self.moveCharacterToStroke(c,stroke,img)
			
	def run(self,path,image):
		if not len(path.strokes):
			raise Exception('No strokes in path "%s"' % path.name)
		self.computePivotY()
		self.initializeCharacters()
		stroke=DirectionStroke(path.strokes[0])
		self.layoutOnStroke(stroke)
		self.moveCharactersToStroke(stroke,image,path.name)
		pdb.gimp_displays_flush()	

def text_along_path(image,layer,path,text,fontName,fontSize,
			layout,pivotYChoice,
			useKerning,extraSpacing,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta):
				
	pdb.gimp_image_undo_group_start(image)
	try:
		# extract data from text layer if provided
		if layer:
			trace('--1--')
			if use28:
				isText=pdb.gimp_drawable_is_text_layer(layer.ID)
			else:
			        isText=pdb.gimp_drawable_is_text_layer(layer)
			if not isText:
				raise Exception('Drawable "%s" is not a text layer' % layer.name)
			trace('--2--')
			text = pdb.gimp_text_layer_get_text(layer)
			if not text:
				text='#TEXT#'
				# raise Exception('No text in layer "%s"' % layer.name)
			trace('--3--')
			fontName = pdb.gimp_text_layer_get_font(layer)
			trace('--4--')
			fontSize, unit = pdb.gimp_text_layer_get_font_size(layer)
			trace("Font: %d, %s" % (fontSize,unit))
			trace('--5--')
			layout = pdb.gimp_text_layer_get_justification(layer)
			trace('--6--')
			useKerning = pdb.gimp_text_layer_get_kerning(layer)
			trace('--7--')
			extraSpacing = pdb.gimp_text_layer_get_letter_spacing(layer)
			trace('--8--')
			
		if not fontName:
			fontName = pdb.gimp_context_get_font()
		if not text:
			raise Exception('No text provided')
		text=Text(text,fontName,fontSize,
				layout,pivotYChoice,
				useKerning, extraSpacing, verticalAdjust,
				keepPaths, keepUpright,
				wiggleXPercent,wiggleYPercent,wiggleTheta)
		text.run(path,image)
	except Exception as e:
		print e
		print dir(e)
		print e.args
		trace(e.args[0])
		pdb.gimp_message(e.args[0])
	pdb.gimp_image_undo_group_end(image)


# Call form path list: no text layer provided, use full options
def text_along_path_full_coin(image,path,text,fontName,fontSize,
			layout,pivotYChoice,
			useKerning,extraSpacing,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta):
	text_along_path(image,None,path,text,fontName,fontSize,
			layout,pivotYChoice,
			useKerning,extraSpacing,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta)
	
# Call from Layers list: many options taken from text layer
def text_along_path_layer(image,layer,path,
			pivotYChoice,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta):
	text_along_path(image,layer,path,None,None,None,
			None,pivotYChoice,
			None,None,verticalAdjust,
			keepPaths, keepUpright,
			wiggleXPercent,wiggleYPercent,wiggleTheta)
#End text-along-path-04 by Ofnuts ############################################################################			
"""
###########################################
# BA_Conbagui_Gold Coin Tutorial Scripted #
###########################################
https://gimplearn.net/viewtopic.php?f=3&t=744&hilit=Gold+Coin+Tutorial
Needs scripts
layerfx bevel/emboss
text_along_path

Also requires 2 vectors and a brush include
in the package.
Put the brush into your brushes folder,
Put the 2 svg path files into a folder you
can easily browse to from the dialog.

file: CoinEdge.gbr < the brush
file: a_path.svg   < the paths to import into the script
file: b_path.svg   < through the dialog box.
...................................................... 
####################
# COPYRIGHT NOTICE #
###############################################################################
# You should have received a copy of the GNU General Public License           #
# along with this program if not, you can view the GNU General Public         #
# License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html  #
# Alternatively you can write to the Free Software Foundation, Inc., 675 Mass #
# Ave, Cambridge, MA 02139, USA.                                              #
###############################################################################
"""



def BA_Conbagui_Coin_gsn(fg_1,fg_2,edg,st,font_t,fs_t,ex_t,text_t,font_b,fs_b,ex_b,text_b,bump,bh_1,bh_2,bh_3,ira,guides):


    white='#ffffff'
    gray ='#ffffff'
    black='#000000'
    pat1 = (_("Clipboard Image")) ## fx_bevel: needs pattern but doesn't use it.  Everyone has the clipboard so no errors.

    image = gimp.Image(1600, 1200, RGB)
    drawable = gimp.Layer(image, "BA_Coin", 1600, 1200, RGBA_IMAGE, 100, 0)
    image.add_layer(drawable, 1)
    gimp.Display(image) 
	
    pdb.gimp_context_push()
    inwidth = image.width
    inheight = image.height
    intype = image.base_type
    inname = drawable.name
    inlays = image.layers
    inimglay0 = inlays[0]
	

    #image = pdb.gimp_image_new (inwidth, inheight, intype)
    #pdb.gimp_display_new(image)  #toggle off for speed and uncluttered display/on for troubleshooting#
    #blackleather = pdb.gimp_layer_new_from_drawable(inimglay0,image) 
    #blackleather.mode = NORMAL_MODE
    #blackleather.name = 'blackleather'
    #blackleather.opacity = 100.0 
    #image.add_layer (blackleather, 0)
    #pdb.gimp_item_set_visible(blackleather, False)  
    #alpha=pdb.gimp_drawable_has_alpha(blackleather)
    #if not alpha:
    #   pdb.gimp_layer_add_alpha(blackleather)    #  RGBA
 
####### make guides for user

    if guides: # creating guides, if chosen by user. Script has no use for them.
        hguide = pdb.gimp_image_add_hguide(image,inheight/2)
        vguide = pdb.gimp_image_add_vguide(image,inwidth/2)
        hguide = pdb.gimp_image_add_hguide(image,50)
        hguide = pdb.gimp_image_add_hguide(image,1150)
        vguide = pdb.gimp_image_add_vguide(image,395) 
        vguide = pdb.gimp_image_add_vguide(image,1205)


####### coin layer
    pdb.gimp_image_select_ellipse(image,0,(inwidth-1000)/2,(inheight-1000)/2, 1000, 1000)
    coinblank = image.new_layer()
    coinblank.name='coin blank'
    pdb.plug_in_sel2path(image, coinblank)
    coinvector = pdb.gimp_image_get_active_vectors(image)
    pdb.gimp_item_set_name(coinvector,'outer coin path')
    pdb.gimp_context_set_foreground(fg_1)
    pdb.gimp_edit_fill(coinblank, 0)
    coin = pdb.gimp_layer_new_from_visible(image,image,'Coin')
    image.add_layer(coin,0)
    pdb.gimp_item_set_visible(coinblank, False)
    pdb.gimp_item_set_visible(coin, False)
    

####### rim layer
    pdb.gimp_selection_shrink(image,7)
    pdb.plug_in_sel2path(image, coinblank) #drawable is unused, but needs to be there.
    edgevector = pdb.gimp_image_get_active_vectors(image)
    pdb.gimp_item_set_name(edgevector,'rim edge vector')
    pdb.gimp_selection_none(image)

    pdb.gimp_image_select_item(image,0,coin)    
    pdb.gimp_context_set_foreground(fg_2)
    rim = image.new_layer()
    rim.name = 'rim'
    pdb.gimp_edit_fill(rim, 0)
    pdb.gimp_selection_shrink(image, 25) #for a total of 25, less the 7 from the edge vector
    pdb.gimp_edit_cut(rim)
    pdb.gimp_selection_none(image)

####### fx bevel the rim of the coin edge

    pdb.python_layer_fx_bevel_emboss(image,rim,1,65,0,10,14,145,35,0,white,5,100.0,black,3,55.0,0,0,pat1,100,100,0,1)
    rim = pdb.gimp_image_get_active_layer(image) # layer_fx changes input layer id - get new id with same name.
    pdb.gimp_item_set_visible(rim, False)
    ############################################################################################
    # TOTAL FAILURE SROKING PATH WITH DIRECTION AND/OR DIRECTION DYNAMIC. GIMP BUG OR OMISSION #
    # WORK AROUND - stroked manually from UI, then made a .gbr Brush from the pattern.         #
    ############################################################################################

####### grooves layer

    grooves=image.new_layer()
    grooves.name = 'grooves'
    if edg:
        pdb.gimp_context_set_foreground(fg_2)
        pdb.gimp_context_set_brush('Coin Edge')
        pdb.gimp_context_set_brush_size(1600)
        pdb.gimp_paintbrush_default(grooves,2,(800,600))
        pdb.gimp_item_set_visible(grooves, False)

####### inner ring layer
    
    innerring=image.new_layer()
    innerring.name='Inner Ring'
    pdb.gimp_context_set_foreground(fg_1)
    pdb.gimp_context_set_brush('Ring Hardness 100')
    pdb.gimp_context_set_brush_size(700)
    pdb.gimp_paintbrush_default(innerring,2, (800,600))
    pdb.plug_in_gauss_rle2(image,innerring,5,5)
# duplicate 
    innerring2 = pdb.gimp_layer_new_from_visible(image,image,'Inner Ring Deep')
    image.add_layer(innerring2,0)
    #cut center out to 560x560 - middle looks deep if ring emboss is lowlook deep in the middle
    pdb.gimp_image_select_ellipse(image,0,520,320,560,560)
    pdb.gimp_edit_cut(innerring2)
    pdb.gimp_item_set_visible(innerring , False)
    pdb.gimp_item_set_visible(innerring2, False)

####### design layer

    design=image.new_layer()
    design.name = 'Design'
    pdb.gimp_image_select_ellipse(image,0,(inwidth-615)/2,(inheight-615)/2, 615, 615)
    pdb.plug_in_sel2path(image,None)
    designvector = pdb.gimp_image_get_active_vectors(image)
    pdb.gimp_item_set_name(designvector,'design path')
    pdb.gimp_selection_none(image)
    pdb.gimp_item_set_visible(design, False)

####### making the design
    if st:
       pdb.gimp_context_set_foreground(fg_1)
       pdb.gimp_context_set_brush('Stars design')
       pdb.gimp_context_set_brush_size(695)
       pdb.gimp_paintbrush_default(design,2,(800,600))    
       pdb.plug_in_gauss_rle2(image,design,5,5)
       pdb.gimp_item_set_visible(design, False) 
####### legend layer

    pdb.gimp_image_set_active_layer(image,design)
    legend=image.new_layer()
    legend.name = 'Legend'
    pdb.gimp_item_set_visible(legend, False)   

####### import the UI created paths in svg file format

	
    a_path = pdb.gimp_vectors_new(image,"a_path")
    pdb.gimp_image_insert_vectors(image, a_path, None, -1)
    pdb.gimp_vectors_stroke_new_from_points(a_path, 0, 12, (395,600, 395,600, 395,50,  1205,50, 1205,600,1205,600  ), 0)


    b_path = pdb.gimp_vectors_new(image,"b_path")
    pdb.gimp_image_insert_vectors(image, b_path, None, -1)
    pdb.gimp_vectors_stroke_new_from_points(b_path, 0, 12, (395,600, 395,600, 395,1150,   1205,1150, 1205,600, 1205,600 ), 0)
	
####### running ofnuts text along path script 
    t_text=text_t.encode("utf-8")
    b_text=text_b.encode("utf-8")

    # pdb.python_fu_text_along_path_full(image, path, text, fontName, fontSize, layout, pivotY, useKerning, extraSpacing, and 6 more 0,0,0,0,0,0)
    text_along_path_full_coin(image,a_path,t_text,font_t,fs_t,2,5,1,ex_t,0,0,0,0,0,0)
    a_stroked = pdb.gimp_image_get_active_vectors(image)
    text_along_path_full_coin(image,b_path,b_text,font_b,fs_b,2,5,1,ex_b,0,0,0,0,0,0)
    b_stroked = pdb.gimp_image_get_active_vectors(image)
    pdb.gimp_selection_none(image)
    
####### set selections for fill and bump

   #pdb.gimp_vectors_to_selection(vectors, operation, antialias, feather, feather_radius_x, feather_radius_y)
    pdb.gimp_vectors_to_selection(a_stroked,0,1,0,0,0)
    pdb.gimp_vectors_to_selection(b_stroked,0,1,0,0,0)
    pdb.gimp_image_set_active_layer(image, legend)
    pdb.gimp_context_set_foreground(gray) #med-light gray for better bump.
    pdb.gimp_edit_fill(legend,0)
    pdb.gimp_selection_none(image)
    pdb.plug_in_gauss_rle2(image,legend,5,5) #medium blur for less alias on bump. 
    pdb.gimp_vectors_set_visible(a_stroked,0)
    pdb.gimp_vectors_set_visible(b_stroked,0)

####### leaving the map layers, if user wants to 'flatten out' the bumped layer 
####### they can make visible the design or inner ring layers for a different look

    if bump != 0:
        pdb.plug_in_bump_map(image,coin,legend,      145,45,bh_3,0,0,0,150,1,0,0)
        pdb.plug_in_bump_map(image,coin,innerring,   145,45,bh_1,0,0,0,ira,1,0,0)
        pdb.plug_in_bump_map(image,coin,design,      145,45,bh_2,0,0,0,150,1,0,0)
        pdb.plug_in_bump_map(image,innerring ,design,145,45,bh_2,0,0,0,150,1,0,0)
        pdb.plug_in_bump_map(image,innerring2,design,145,45,bh_2,0,0,0,150,1,0,0)

####### Cleanup and exit.
    #put color back on legend
    pdb.gimp_image_select_item(image,0,legend)
    pdb.gimp_context_set_foreground(fg_1)
    pdb.gimp_edit_fill(legend,0)
    pdb.gimp_selection_none(image)
    #set visibles
    pdb.gimp_item_set_visible(drawable,True)
    pdb.gimp_item_set_visible(coin,True)
    pdb.gimp_item_set_visible(rim,True)
    pdb.gimp_item_set_visible(grooves,True)

    #pdb.gimp_display_new(image) #swap this with the display new in 1st layer creation when script is completed.
    pdb.gimp_displays_flush()    
    pdb.gimp_context_pop()
    return()    

# This is the plugin registration function
register(
    "BA_Conbagui_Coin_gsn",    
    "Script Based On Conbagui's Gold Coin Tutorial \nThis Plugin Create New Image 1600 x 1200 ",   
    "Requirments show at top",
    "animicule@gimpchat.com", 
    "Brian Andersen/MrQ", 
    "2017/2020",
    "BA Gold Coin...", 
    "", 
   [
      #(PF_IMAGE,    "image",     "Input image             ",None),
      #(PF_DRAWABLE, "idrawable", "Input drawable          ",None),
      #(PF_DIRNAME,  "vectordir", "Saved Vectors Directory ", " "),
      (PF_COLOR,    "fg_1",      "Pick Main Color         ",(206,187,120)),
      (PF_COLOR,    "fg_2",      "Pick Rim Color          ",(138,124, 76)),
      (PF_TOGGLE,   "edg",       "Make Rim Grooves?       ", 1),
      (PF_TOGGLE,   "st",        "Make Stars?             ", 1),	  
      (PF_FONT,     "font_t",    "Top Legend Font         ", "Sans-serif"),
      #(PF_INT,      "fs_t",      "Top Legend Font Size    ", 100),
      (PF_SPINNER, "fs_t",       "Top Legend Font Size:", 100, (0, 200, 1, 5, 0, 0)),	  
      (PF_INT,      "ex_t",      "Extra Spacing Top Font  ", 3),
      (PF_STRING,   "text_t",    "Top Legend Text         ", "★ GIMPSCRIPTS.NET ★"),
      (PF_FONT,     "font_b",    "Bottom Font             ", "Sans-serif"),
      #(PF_INT,      "fs_b",      "Bottom Font Size        ", 80),
      (PF_SPINNER, "fs_b",       "Bottom Font Size",      100, (0, 200, 1, 5, 0, 0)),		  
      (PF_INT,      "ex_b",      "Extra Spacing Bottom    ", 3),
      (PF_STRING,   "text_b",    "Top Legend Text         ", "☣ GIMPSCRIPTS.NET ☣"),
      (PF_BOOL,     "bump",      "Bump Map                ", True),
      #(PF_INT,      "bh_1",      "Bump Height Rim         ", 40),
      (PF_SPINNER, "bh_1",       "Bump Height Rim",        45, (1, 65, 1, 5, 0, 0)),	  
      #(PF_INT,      "bh_2",      "Bump Height Stars       ", 40),
      (PF_SPINNER, "bh_2",       "Bump Height Stars",      45, (1, 65, 1, 5, 0, 0)),  
      #(PF_INT,      "bh_3",      "Bump Height Legend      ", 40),
      (PF_SPINNER, "bh_2",       "Bump Height Legend",      45, (1, 65, 1, 5, 0, 0)),	  
      #(PF_INT,      "ira",       "Bump Ambient InnerRing  ",150),
      (PF_SPINNER, "ira",       "Bump Ambient InnerRing",      150, (0, 255, 1, 5, 0, 0)),	  
      (PF_BOOL,     "guides",    "Show Guides             ", True),            
    ], 
    [],
    BA_Conbagui_Coin_gsn,
    menu="<Image>/Filters/Render",
    domain=("gimp20", gimp.locale_directory)
    )

main()
